/**
  EncodePost - Base64 encode/decode fields in posted forms
  Written by (c) Petko Yotov 2023    www.pmwiki.org/petko

  This text is written for PmWiki; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version. See pmwiki.php for full details
  and lack of warranty.

  Version 20230504
*/

document.addEventListener('DOMContentLoaded', function(){
  if(!window.btoa || !window.unescape) return;

  var pmtext = document.querySelector('textarea#text[name="text"]');
  var atop = document.querySelector('top');

  if(!pmtext) return;

  var form = pmtext.form;
  var buttons = form.querySelectorAll('input[type="submit"]');
  for(var i=0; i<buttons.length; i++) {
    buttons[i].addEventListener('click', submitForm);
  }

  function submitForm(e){
    e.preventDefault();
    if(this.name == 'cancel' && this.dataset.url) {
      self.location.href = this.dataset.url;
      return;
    }
    
    var p = document.getElementById('wikipreview');
    if(p) {
      p.parentNode.removeChild(p);
      if(atop) {
        top.scrollIntoView({block: "start", inline: "nearest", behavior: "smooth"});
      }
    }
    var fields = form.querySelectorAll('input[name],textarea[name]');

    var fd = new FormData();
    for(var i=0; i<fields.length; i++) {
      if(fields[i].type == 'submit' && fields[i] !== this) continue;
      if(!fields[i].reportValidity()) return;
      if(fields[i].type.match(/^(checkbox|radio)$/) && ! fields[i].checked) continue;

      var n = fields[i].name, v = fields[i].value;
      if(n=='EXtext') continue;
      if(n=="text") {
        if(typeof EXparts != 'undefined' && EXparts.length) v = EXparts.join('\n').replace(/^\n+/, '');
        if(v) v = b64(v);
      }
      if(n=='action') v = 'encodedpost';
      fd.append(n, v);
    }

    var opt = {
      method: 'POST',
      body: fd
    }
    var url = form.getAttribute('action').replace(/\b(action=)[-\w]+/, 'action=encodedpost');

    fetch(url, opt).then(function(r){
      if(! r.ok) return null;
      return r.json();
    }).then(function(j){
      if(!j) {
        return alert("Undefined error");
      }
      if(j.failure) {
        return alert("Error, message is: \n" + j.failure);
      }
      if(j.preview) {
        var wf = document.getElementById('wikiedit');
        wf.insertAdjacentHTML('beforebegin', j.preview);
      }
      if(j.redirect) {
        for(var i=0; i<fields.length; i++) {
          fields[i].defaultValue = fields[i].value;
        }
        self.location.href = j.redirect;
      }
      else {
        if(j.n) {
          form.n.value = j.n;
          form.n.defaultValue = j.n;
        }
        if(j.basetime) {
          form.basetime.value = j.basetime;
          form.basetime.defaultValue = j.basetime;
        }
      }
    }).catch(function(e){
      console.log({fetcherror:e});
    });
  }

  function b64(str) {
    return 'base64::' + window.btoa(unescape(encodeURIComponent(str)));
  }
});
